package ch.rakudave.jnetmap.view.preferences;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.table.DefaultTableModel;

import ch.rakudave.jnetmap.plugins.RightClickExec;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.logging.Logger;

/**
 * @author rakudave
 */
@SuppressWarnings("serial")
public class ScriptsPanel extends PreferencePanel {
	private static boolean dirty;
	private DefaultTableModel tableModel;

	public ScriptsPanel() {
		title = Lang.getNoHTML("preferences.scripts");
		parentTitle = Lang.getNoHTML("preferences.plugins");
		// Table
			String columnNames[] = {
					Lang.get("preferences.scripts.name"),
					Lang.get("preferences.scripts.command"),
					Lang.get("preferences.scripts.args")
			};
			int count = Settings.getInt("plugin.exec.count", 0);
			String data[][] = new String[count][];
			for (int i = 0; i < count; i++) {
				data[i] = new String[] {
						Settings.get("plugin.exec."+i+".name", ""),
						Settings.get("plugin.exec."+i+".command", ""),
						Settings.get("plugin.exec."+i+".args", "")};
			}
			tableModel = new DefaultTableModel(data, columnNames);
			final JTable table = new JTable(tableModel);
		// Buttons
			JPanel buttons = new JPanel(new GridLayout(1, 2));
				JButton addButton = new JButton(new AbstractAction(Lang.get("action.add"), Icons.get("add")) {
					@Override
					public void actionPerformed(ActionEvent e) {
						tableModel.addRow(new String[3]);
					}
				});
				JButton removeButton = new JButton(new AbstractAction(Lang.get("action.remove"), Icons.get("remove")) {
					@Override
					public void actionPerformed(ActionEvent e) {
						int row = table.getSelectedRow();
						if (row > -1) tableModel.removeRow(row);
					}
				});
				buttons.add(addButton);
				buttons.add(removeButton);
		// Legend
			JTextPane legend = new JTextPane();
				legend.setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.scripts.args")));
				legend.setMaximumSize(new Dimension(500, 25));
				legend.setFocusable(false);
				legend.setContentType("text/html");
				legend.setBackground(new Color(getBackground().getRGB()));
				StringBuffer sb = new StringBuffer("<html><table width=\"100%\">");
					sb.append("<tr><td>%n</td><td>").append(Lang.getNoHTML("device.name")).append("</td>");
					sb.append("<td>%d</td><td>").append(Lang.getNoHTML("device.description")).append("</td></tr>");
					sb.append("<tr><td>%a</td><td>").append(Lang.getNoHTML("interface.address")+" (IP)").append("</td>");
					sb.append("<td>%l</td><td>").append(Lang.getNoHTML("device.location")).append("</td></tr>");
					sb.append("<tr><td>%h</td><td>").append(Lang.getNoHTML("interface.address")+" (Host)").append("</td>");
					sb.append("<td>%m</td><td>").append(Lang.getNoHTML("device.model")).append("</td></tr>");
					sb.append("<tr><td>%s</td><td>").append(Lang.getNoHTML("message.status")).append("</td>");
					sb.append("<td>%v</td><td>").append(Lang.getNoHTML("device.vendor")).append("</td></tr>");
					sb.append("<tr><td>%t</td><td>").append(Lang.getNoHTML("device.type")).append("</td>");
					sb.append("<td>%i</td><td>").append(Lang.getNoHTML("device.icon")).append("</td></tr></table></html>");
				legend.setText(sb.toString());
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(new JScrollPane(table, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
		add(buttons);
		add(Box.createVerticalGlue());
		add(legend);
	}

	@Override
	public void save() {
		int count = 0;
		for (int i = 0; i < tableModel.getRowCount(); ++i) {
			try {
				Object c1 = tableModel.getValueAt(i, 0), c2 = tableModel.getValueAt(i, 1), c3 = tableModel.getValueAt(i, 2);
				if (c1 != null && c2 != null) {
					Settings.put("plugin.exec."+count+".name", c1.toString());
					Settings.put("plugin.exec."+count+".command", c2.toString());
					if (c3 != null) Settings.put("plugin.exec."+count+".args", c3.toString());
					++count;
				} else {
					Logger.warn("Skipped script on line "+i+" because name or command is missing");
				}
			} catch (Exception e) {
				Logger.warn("Failed to save script on line "+i, e);
			}
		}
		Settings.put("plugin.exec.count", count);
		dirty = true;
	}
	
	public static List<RightClickExec> getScriptPlugins() {
		int count = Settings.getInt("plugin.exec.count", 0);
		List<RightClickExec> result = new LinkedList<RightClickExec>();
		for (int i = 0; i < count; i++) {
			try {
				RightClickExec p = new RightClickExec(
						Settings.get("plugin.exec."+i+".name", ""),
						Settings.get("plugin.exec."+i+".command", ""),
						Settings.get("plugin.exec."+i+".args", ""));
				Logger.debug("Adding exec plugin "+p.toString());
				result.add(p);
			} catch (Exception e) {
				Logger.debug("Failed to add exec plugin #"+i);
			}
		}
		return result;
	}
	
	public static boolean isDirty() {
		return dirty;
	}
}
